/* Copyright 2009 ObjectFactory (http://www.wingnest.com/softs/grails-plugins)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
package org.codehaus.groovy.grails.plugins.appBean;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import groovy.util.ConfigObject;
import org.codehaus.groovy.grails.commons.ApplicationHolder;
import org.codehaus.groovy.grails.plugins.appBean.annotation.AppBean;
import org.codehaus.groovy.grails.plugins.appBean.annotation.AppCustomBean;

/**
 * @author Shigeru GOUGI
 */
final public class AppBeanUtils {
    private static Map populatedBeanMap = null;

    static {
        try {
            populateBeanMap();
        } catch (MalformedURLException e) {
            e.printStackTrace();
            Runtime.getRuntime().exit(-1);
        }
    }

    public static String getCustomBeanName(Class clazz) {
        String name = clazz.getName();
        String role = (String) populatedBeanMap.get(name);
        return role;
    }

    public static AppBean getAppBeanAnnotation(Class<?> clazz) {
        assert clazz != null;
        return (AppBean) clazz.getAnnotation(AppBean.class);
    }

    public static AppCustomBean getAppCustomBeanAnnotation(Class<?> clazz) {
        assert clazz != null;
        return (AppCustomBean) clazz.getAnnotation(AppCustomBean.class);
    }

    //////////////////////////////////////////////////////////
    private static void populateBeanMap() throws MalformedURLException {
        final ConfigObject configObject = ApplicationHolder.getApplication().getConfig();
        populatedBeanMap = new HashMap<Object, Object>();
        final Map customBeanMap = (Map) ((ConfigObject) configObject.getProperty("appBean")).getProperty("customBeanMap");
        if( customBeanMap == null ) {
            return;
        }
        final Iterator it = customBeanMap.entrySet().iterator();
        if (it.hasNext()) {
            do {
                final Entry ent = (Entry) it.next();
                populatedBeanMap.put(ent.getValue(), ent.getKey());
            } while (it.hasNext());
        }
    }

}
